pub trait GradientType<OutputType> {
type GradientType;
}Expand description
Compile-time calculation for what the gradient type should be based on input and output types.
For most cases, the gradient type is the same as the output type. However, for multi-parameter functions, like functions of arrays, this is not the case
<InputType as GradientType<OutputType>>::GradientType
Required Associated Types
Sourcetype GradientType
type GradientType
The type of the gradient for a function with input type Self and output type OutputType